import click

from mvf1 import MultiViewerForF1
from mvf1 import MultiViewerForF1Error

remote = MultiViewerForF1()

@click.group()
def cli():
    pass


@cli.group(help="Query and control MultiViewerForF1 players.")
def players():
    pass

@players.command(help="List all active MultiViewerForF1 players.")
@click.option('--verbose', is_flag=True, default=False,
              help="Display all player information.")
def ls(verbose):
   players = remote.players

   if len(players) == 0:
       raise click.UsageError("No active players found. Is MultiViewerForF1 running?")

   for player in players:
       click.echo(f"ID: {player.id} - Title: {player.title}")
       if verbose:
           click.echo("--------------------------------------------------")
           click.echo(f"State: {player.state}")
           click.echo(f"driverData: {player.driver_data}")
           click.echo(f"streamData: {player.stream_data}")
           click.echo(f"Position: {player.x}, {player.y}")
           click.echo(f"Dimensions: {player.width}, {player.height}")
           click.echo(f"Fullscreen: {player.fullscreen}")
           click.echo(f"Always On Top: {player.always_on_top}")
           click.echo(f"Maintain Aspect Ratio: {player.maintain_aspect_ratio}")

@players.command(help="Close all active MultiViewerForF1 players.",
                 name="close")
def players_close():
    players = remote.players
    for player in players:
        click.echo(f"Closing ID {player.id} - {player.title}...")
        player.delete()
    click.echo("Done.")

@players.command(help="Mute all active MultiViewerForF1 players.",
                 name="mute")
def players_mute():
    players = remote.players
    for player in players:
        click.echo(f"Muting ID {player.id} - {player.title}...")
        player.mute()
    click.echo("Done.")

@players.command(help="Pause all active MultiViewerForF1 players.",
                 name="pause")
def players_pause():
    players = remote.players
    for player in players:
        click.echo(f"Pausing ID {player.id} - {player.title}...")
        player.pause()
    click.echo("Done.")

@players.command(help="Synchronize all active MultiViewerForF1 players to the "
                 "player streaming broadcast commentary.",
                 name="sync")
def players_sync():
    click.echo("Syncing all players to broadcast commentary...")
    remote.player_sync_to_commentary()
    click.echo("Done.")

@cli.group(help="Query and control a specific MultiViewerForF1 player.")
@click.option('--id', required=False, type=int)
@click.option('--title', required=False, type=str)
@click.pass_context
def player(ctx, id, title):
    if not id and not title:
        raise click.UsageError("To control a player, pass either an --id or "
                               "--title. To see IDs and Titles, use mvf1 "
                               "players ls")
    ctx.ensure_object(dict)

    if id:
        try:
            ctx.obj['player'] = remote.player(id)
        except MultiViewerForF1Error as e:
            raise click.UsageError(e)
    else:
        players = remote.players

        for player in players:
            if player.title == title:
                ctx.obj['player'] = player
                return
        raise click.UsageError("No player found with title "
                               f"{title}")

@player.command(help="Display information about player.")
@click.option('--verbose', is_flag=True, default=False,
              help="Display all player information.")
@click.pass_context
def query(ctx, verbose):
    player = ctx.obj['player']

    click.echo(f"ID: {player.id} - Title: {player.title}")
    if verbose:
        click.echo("--------------------------------------------------")
        click.echo(f"State: {player.state}")
        click.echo(f"driverData: {player.driver_data}")
        click.echo(f"streamData: {player.stream_data}")
        click.echo(f"Position: {player.x}, {player.y}")
        click.echo(f"Dimensions: {player.width}, {player.height}")
        click.echo(f"Fullscreen: {player.fullscreen}")
        click.echo(f"Always On Top: {player.always_on_top}")
        click.echo(f"Maintain Aspect Ratio: {player.maintain_aspect_ratio}")

@player.command(help="Close player",
                name="close")
@click.pass_context
def player_close(ctx):
    player = ctx.obj['player']
    click.echo(f"Closing player ID: {player.id} - {player.title}...")
    player.delete()
    click.echo("Done.")

@player.command(help="Seek to relative or absolute position")
@click.option('--absolute', required=False, type=int)
@click.option('--relative', required=False, type=int)
@click.pass_context
def seek(ctx, absolute, relative):
    if not absolute and not relative:
        raise click.UsageError("To seek a player, pass with an --absolute or "
                               "--relative number of seconds. This can any positive"
                               "or negative integer.")

    player = ctx.obj['player']
    click.echo(f"Seeking player ID {player.id} - {player.title}...")
    player.seek(absolute=absolute, relative=relative)
    click.echo("Done.")

@player.command(help="Adjust position and/or dimensions")
@click.option('--x', required=False, type=int)
@click.option('--y', required=False, type=int)
@click.option('--width', required=False, type=int)
@click.option('--height', required=False, type=int)
@click.pass_context
def set_bounds(ctx, x, y, width, height):
    if not x and not y and not width and not height:
        raise click.UsageError("To adjust position or dimensions of a player, "
                               "pass --x and --y to place the top left corner "
                               "of the player. Pass --width and --height for "
                               "the dimensions of the player.")

    player = ctx.obj['player']
    click.echo(f"Adjusting bounds of player ID {player.id} - "
               f"{player.title}...")
    player.set_bounds(x=x, y=y, width=width, height=height)
    click.echo("Done.")

@player.command(help="Adjust volume")
@click.option('--volume', required=True, type=int)
@click.pass_context
def set_volume(ctx, volume):
    if not volume:
        raise click.UsageError("To adjust volume, pass --volume with an "
                               "integer between 1 and 100.")
    player = ctx.obj['player']
    click.echo(f"Setting volume for player ID {player.id} - "
               f"{player.title}...")
    player.set_volume(volume=volume)
    click.echo("Done.")

@player.command(help="Pause")
@click.pass_context
def pause(ctx):
    player = ctx.obj['player']
    click.echo(f"Pausing player ID {player.id} - {player.title}...")
    player.pause()
    click.echo("Done.")

@player.command(help="Toggle fullscreen")
@click.pass_context
def set_fullscreen(ctx):
    player = ctx.obj['player']
    click.echo(f"Toggling fullscreen for player ID {player.id} - "
               f"{player.title}...")
    player.set_fullscreen()
    click.echo("Done.")

@player.command(help="Mute")
@click.pass_context
def mute(ctx):
    player = ctx.obj['player']
    click.echo(f"Muting player ID {player.id} - {player.title}...")
    player.mute()
    click.echo("Done.")

@player.command(help="Toggle speedometer visibility")
@click.pass_context
def set_speedometer_visibility(ctx):
    player = ctx.obj['player']
    click.echo(f"Toggling speedometer visibility player ID {player.id} - "
               f"{player.title}...")
    player.set_speedometer_visibility()
    click.echo("Done.")

@player.command(help="Synchronize all players to this player's timestamp")
@click.pass_context
def sync(ctx):
    player = ctx.obj['player']
    click.echo(f"Synchronizing all players to player ID {player.id} - "
               f"{player.title}...")
    player.sync()
    click.echo("Done.")

@player.command(help="Switch player's video stream.")
@click.option('--title', required=True, type=str)
@click.pass_context
def switch_stream(ctx, title):
    if not title:
        raise click.UsageError("To switch the stream of a player, pass the "
                               "--title of the desired stream.\n\n"
                               "For example, use INTERNATIONAL for Crofty or "
                               "the three letter acronoym for a driver - "
                               "PER for Sergio Perez.")
    player = ctx.obj['player']
    click.echo(f"Switching stream for player ID {player.id} - {player.title} "
               f"to {title}...")
    player.switch_stream(title)
    click.echo("Done.")



import sgqlc.types


mvf1_schema = sgqlc.types.Schema()



########################################################################
# Scalars and Enumerations
########################################################################
class BigInt(sgqlc.types.Scalar):
    __schema__ = mvf1_schema


Boolean = sgqlc.types.Boolean

Float = sgqlc.types.Float

ID = sgqlc.types.ID

Int = sgqlc.types.Int

class JSONObject(sgqlc.types.Scalar):
    __schema__ = mvf1_schema


class PlayerType(sgqlc.types.Enum):
    __schema__ = mvf1_schema
    __choices__ = ('ADDITIONAL', 'OBC')


String = sgqlc.types.String


########################################################################
# Input Objects
########################################################################
class PlayerCreateInput(sgqlc.types.Input):
    __schema__ = mvf1_schema
    __field_names__ = ('content_id', 'channel_id', 'driver_tla', 'driver_number', 'stream_title', 'bounds', 'fullscreen', 'always_on_top', 'maintain_aspect_ratio')
    content_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='contentId')
    channel_id = sgqlc.types.Field(Int, graphql_name='channelId')
    driver_tla = sgqlc.types.Field(String, graphql_name='driverTla')
    driver_number = sgqlc.types.Field(Int, graphql_name='driverNumber')
    stream_title = sgqlc.types.Field(String, graphql_name='streamTitle')
    bounds = sgqlc.types.Field('RectangleInput', graphql_name='bounds')
    fullscreen = sgqlc.types.Field(Boolean, graphql_name='fullscreen')
    always_on_top = sgqlc.types.Field(Boolean, graphql_name='alwaysOnTop')
    maintain_aspect_ratio = sgqlc.types.Field(Boolean, graphql_name='maintainAspectRatio')


class RectangleInput(sgqlc.types.Input):
    __schema__ = mvf1_schema
    __field_names__ = ('x', 'y', 'width', 'height')
    x = sgqlc.types.Field(Int, graphql_name='x')
    y = sgqlc.types.Field(Int, graphql_name='y')
    width = sgqlc.types.Field(Int, graphql_name='width')
    height = sgqlc.types.Field(Int, graphql_name='height')



########################################################################
# Output Objects and Interfaces
########################################################################
class LiveTimingClock(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('paused', 'system_time', 'track_time', 'live_timing_start_time')
    paused = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='paused')
    system_time = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='systemTime')
    track_time = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='trackTime')
    live_timing_start_time = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='liveTimingStartTime')


class LiveTimingState(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('archive_status', 'audio_streams', 'car_data', 'championship_prediction', 'content_streams', 'driver_list', 'extrapolated_clock', 'heartbeat', 'lap_count', 'lap_series', 'pit_lane_time_collection', 'position', 'race_control_messages', 'session_data', 'session_info', 'session_status', 'team_radio', 'timing_app_data', 'timing_data', 'timing_stats', 'top_three', 'track_status', 'weather_data', 'weather_data_series')
    archive_status = sgqlc.types.Field(JSONObject, graphql_name='ArchiveStatus')
    audio_streams = sgqlc.types.Field(JSONObject, graphql_name='AudioStreams')
    car_data = sgqlc.types.Field(JSONObject, graphql_name='CarData')
    championship_prediction = sgqlc.types.Field(JSONObject, graphql_name='ChampionshipPrediction')
    content_streams = sgqlc.types.Field(JSONObject, graphql_name='ContentStreams')
    driver_list = sgqlc.types.Field(JSONObject, graphql_name='DriverList')
    extrapolated_clock = sgqlc.types.Field(JSONObject, graphql_name='ExtrapolatedClock')
    heartbeat = sgqlc.types.Field(JSONObject, graphql_name='Heartbeat')
    lap_count = sgqlc.types.Field(JSONObject, graphql_name='LapCount')
    lap_series = sgqlc.types.Field(JSONObject, graphql_name='LapSeries')
    pit_lane_time_collection = sgqlc.types.Field(JSONObject, graphql_name='PitLaneTimeCollection')
    position = sgqlc.types.Field(JSONObject, graphql_name='Position')
    race_control_messages = sgqlc.types.Field(JSONObject, graphql_name='RaceControlMessages')
    session_data = sgqlc.types.Field(JSONObject, graphql_name='SessionData')
    session_info = sgqlc.types.Field(JSONObject, graphql_name='SessionInfo')
    session_status = sgqlc.types.Field(JSONObject, graphql_name='SessionStatus')
    team_radio = sgqlc.types.Field(JSONObject, graphql_name='TeamRadio')
    timing_app_data = sgqlc.types.Field(JSONObject, graphql_name='TimingAppData')
    timing_data = sgqlc.types.Field(JSONObject, graphql_name='TimingData')
    timing_stats = sgqlc.types.Field(JSONObject, graphql_name='TimingStats')
    top_three = sgqlc.types.Field(JSONObject, graphql_name='TopThree')
    track_status = sgqlc.types.Field(JSONObject, graphql_name='TrackStatus')
    weather_data = sgqlc.types.Field(JSONObject, graphql_name='WeatherData')
    weather_data_series = sgqlc.types.Field(JSONObject, graphql_name='WeatherDataSeries')


class Mutation(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('version', 'player_create', 'player_delete', 'player_set_fullscreen', 'player_set_bounds', 'player_set_volume', 'player_set_paused', 'player_set_muted', 'player_seek_to', 'player_sync', 'player_set_speedometer_visibility')
    version = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='version')
    player_create = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='playerCreate', args=sgqlc.types.ArgDict((
        ('input', sgqlc.types.Arg(sgqlc.types.non_null(PlayerCreateInput), graphql_name='input', default=None)),
))
    )
    player_delete = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerDelete', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
))
    )
    player_set_fullscreen = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSetFullscreen', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('fullscreen', sgqlc.types.Arg(Boolean, graphql_name='fullscreen', default=None)),
))
    )
    player_set_bounds = sgqlc.types.Field(sgqlc.types.non_null('Rectangle'), graphql_name='playerSetBounds', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('bounds', sgqlc.types.Arg(sgqlc.types.non_null(RectangleInput), graphql_name='bounds', default=None)),
))
    )
    player_set_volume = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='playerSetVolume', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('volume', sgqlc.types.Arg(sgqlc.types.non_null(Float), graphql_name='volume', default=None)),
))
    )
    player_set_paused = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSetPaused', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('paused', sgqlc.types.Arg(Boolean, graphql_name='paused', default=None)),
))
    )
    player_set_muted = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSetMuted', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('muted', sgqlc.types.Arg(Boolean, graphql_name='muted', default=None)),
))
    )
    player_seek_to = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='playerSeekTo', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('absolute', sgqlc.types.Arg(Float, graphql_name='absolute', default=None)),
        ('relative', sgqlc.types.Arg(Float, graphql_name='relative', default=None)),
))
    )
    player_sync = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSync', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
))
    )
    player_set_speedometer_visibility = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSetSpeedometerVisibility', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('visible', sgqlc.types.Arg(Boolean, graphql_name='visible', default=None)),
))
    )


class Player(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('id', 'type', 'state', 'driver_data', 'stream_data', 'bounds', 'fullscreen', 'always_on_top', 'maintain_aspect_ratio')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    type = sgqlc.types.Field(sgqlc.types.non_null(PlayerType), graphql_name='type')
    state = sgqlc.types.Field('PlayerState', graphql_name='state')
    driver_data = sgqlc.types.Field('PlayerDriverData', graphql_name='driverData')
    stream_data = sgqlc.types.Field('PlayerStreamData', graphql_name='streamData')
    bounds = sgqlc.types.Field(sgqlc.types.non_null('Rectangle'), graphql_name='bounds')
    fullscreen = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='fullscreen')
    always_on_top = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='alwaysOnTop')
    maintain_aspect_ratio = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='maintainAspectRatio')


class PlayerDriverData(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('driver_number', 'tla', 'first_name', 'last_name', 'team_name')
    driver_number = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='driverNumber')
    tla = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='tla')
    first_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='firstName')
    last_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='lastName')
    team_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='teamName')


class PlayerState(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('ts', 'paused', 'muted', 'volume', 'live', 'current_time', 'interpolated_current_time')
    ts = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='ts')
    paused = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='paused')
    muted = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='muted')
    volume = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='volume')
    live = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='live')
    current_time = sgqlc.types.Field(Float, graphql_name='currentTime')
    interpolated_current_time = sgqlc.types.Field(Float, graphql_name='interpolatedCurrentTime')


class PlayerStreamData(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('content_id', 'meeting_key', 'session_key', 'channel_id', 'title')
    content_id = sgqlc.types.Field(ID, graphql_name='contentId')
    meeting_key = sgqlc.types.Field(String, graphql_name='meetingKey')
    session_key = sgqlc.types.Field(String, graphql_name='sessionKey')
    channel_id = sgqlc.types.Field(Int, graphql_name='channelId')
    title = sgqlc.types.Field(String, graphql_name='title')


class Query(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('version', 'system_info', 'players', 'player', 'live_timing_state', 'live_timing_clock')
    version = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='version')
    system_info = sgqlc.types.Field(sgqlc.types.non_null('SystemInfo'), graphql_name='systemInfo')
    players = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Player))), graphql_name='players')
    player = sgqlc.types.Field(sgqlc.types.non_null(Player), graphql_name='player', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
))
    )
    live_timing_state = sgqlc.types.Field(LiveTimingState, graphql_name='liveTimingState')
    live_timing_clock = sgqlc.types.Field(LiveTimingClock, graphql_name='liveTimingClock')


class Rectangle(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('x', 'y', 'width', 'height')
    x = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='x')
    y = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='y')
    width = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='width')
    height = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='height')


class SystemInfo(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('platform', 'arch')
    platform = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='platform')
    arch = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='arch')



########################################################################
# Unions
########################################################################

########################################################################
# Schema Entry Points
########################################################################
mvf1_schema.query_type = Query
mvf1_schema.mutation_type = Mutation
mvf1_schema.subscription_type = None




from .mvf1 import MultiViewerForF1
from .mvf1 import MultiViewerForF1Error
from .mvf1 import Player

__title__ = 'mvf1'
__version__ = '1.0.1'
__author__ = 'Rob Spectre'
__license__ = 'MIT'
__copyright__ = 'Copyright 2023 Rob Spectre'
